@@ -3,8 +3,14 @@ import typing as t
 from functools import singledispatchmethod
 
 import unyt
+from unyt import unyt_array
 from unyt.exceptions import UnitConversionError
 
+try:
+    import xarray as xr
+except ImportError:
+    xr = None
+
 logger = logging.getLogger(__name__)
 
 #===============================================================================
@@ -72,6 +78,7 @@ class UnitInputParser():
                  default_units: UnitLike = None,
                  convert: bool = False,
                  check_dims: bool = False,
+                 copy: bool = True,
                  registry: unyt.UnitRegistry = None):
         """
         Parameters
@@ -87,6 +94,8 @@ class UnitInputParser():
             If True, ensures that input has units compatible with
             `default_units`, but does not convert the input. Has no effect if
             `default_units` is None or `convert` is True. (default: False)
+        copy : bool, optional
+            Whether to copy underlying input data. (default: True)
         registry : unyt.UnitRegistry, optional
             Registry used to construct new unyt_array instances. Necessary if
             the desired units are not in the default unit registry. (default:
@@ -96,7 +105,22 @@ class UnitInputParser():
         self.default_units = default_units
         self.convert = convert
         self.check_dims = check_dims
+        self.copy = copy
 
+    def __repr__(self):
+        clsname = self.__class__.__name__
+        attrs = [
+            f'default_units={self.default_units!r}',
+            f'convert={self.convert!r}',
+            f'check_dims={self.check_dims!r}',
+            f'copy={self.copy!r}',
+            f'registry={self.registry!r}',
+        ]
+        return f'{clsname}(' + ', '.join(attrs) + ')'
+
+    #===========================================================================
+    # Units handling
+    #===========================================================================
     @property
     def default_units(self) -> t.Union[unyt.Unit, None]:
         """Default units to use if inputs don't have units associated already.
@@ -120,10 +144,35 @@ class UnitInputParser():
             units = unyt.Unit(units, registry=self.registry)
         return units
 
-    def __call__(self, in_, units: UnitLike = None) -> unyt.unyt_array:
+    #===========================================================================
+    # Dims checking
+    #===========================================================================
+    def _get_units(self, q) -> unyt.Unit:
+        """Get the units of an object."""
+        try:
+            units = q.units
+        except AttributeError:
+            units = unyt.dimensionless
+        return unyt.Unit(units, registry=self.registry)
+
+    def _check_dimensions(self, a, b):
+        """Check that a and b have the same dimensions, and raise an error if
+        they do not.
+        """
+        units_a = self._get_units(a)
+        units_b = self._get_units(b)
+        dim_a = units_a.dimensions
+        dim_b = units_b.dimensions
+        if dim_a != dim_b:
+            raise UnitConversionError(units_a, dim_a, units_b, dim_b)
+
+    #===========================================================================
+    # Parsing
+    #===========================================================================
+    def __call__(self, in_, units: t.Optional[UnitLike] = None):
         return self.parse(in_, units)
 
-    def parse(self, in_, units: UnitLike = None) -> unyt.unyt_array:
+    def parse(self, in_, units: t.Optional[UnitLike] = None) -> unyt_array:
         """Parse the given input expression.
 
         Accepts the following input styles::
@@ -176,24 +225,36 @@ class UnitInputParser():
         if q.ndim == 0:
             q = q.reshape(1)[0]
 
+        if self.copy:
+            q = q.copy(order='K')
+
         if units is not None:
             # Skip dims check if convert is True, since the same check will
             # happen internally inside unyt.
             if self.check_dims and not self.convert:
-                _check_dimensions(q, units)
+                self._check_dimensions(q, units)
 
             if self.convert:
-                q = q.to(units)
+                q.convert_to_units(units)
 
         return q
 
+    #--------------------------------------------------------
+    # Parse internal
+    #
+    # These methods define how 'parse' processes different
+    # types into a unyt_array. They should never copy input
+    # data, if possible, and they should always return
+    # unyt_array, not unyt_quantity (scalarfication is
+    # handled inside `parse`).
+    #--------------------------------------------------------
     @singledispatchmethod
-    def _parse_internal(self, in_, units=None) -> unyt.unyt_array:
+    def _parse_internal(self, in_, units=None) -> unyt_array:
         if units is None:
             raise ValueError('No default units set; cannot parse object '
                              f'without units {in_!r}')
 
-        return unyt.unyt_array(in_, units, registry=self.registry)
+        return unyt_array(in_, units, registry=self.registry)
 
     @_parse_internal.register
     def _(self, in_: unyt.unyt_array, units=None):
@@ -204,14 +265,23 @@ class UnitInputParser():
         if len(in_) != 2:
             raise ValueError(f'Input tuple must have 2 items (got {len(in_)})')
 
-        return unyt.unyt_array(*in_, registry=self.registry)
+        return unyt_array(*in_, registry=self.registry)
+
+    if xr is not None:
+
+        @_parse_internal.register
+        def _(self, in_: xr.DataArray, units=None):
+            value = in_.values
+            units = in_.attrs.get('units', units)
+            return self._parse_internal(value, units)
 
 
 def process_unit_input(in_,
                        default_units: UnitLike = None,
                        convert: bool = False,
                        check_dims: bool = False,
-                       registry: unyt.UnitRegistry = None) -> unyt.unyt_array:
+                       copy: bool = True,
+                       registry: unyt.UnitRegistry = None) -> unyt_array:
     """Process an input value that may or may not have units.
 
     If the input value doesn't have units, assumes the input is in the requested
@@ -245,6 +315,8 @@ def process_unit_input(in_,
         If True, ensures that input has units compatible with `default_units`,
         but does not convert the input. Has no effect if `default_units` is
         None or `convert` is True. (default: False)
+    copy : bool, optional
+        Whether to copy underlying input data. (default: True)
     registry : unyt.UnitRegistry, optional
         Necessary if the desired units are not in the default unit registry.
         Used to construct the returned unyt.unyt_array object.
@@ -265,28 +337,11 @@ def process_unit_input(in_,
     parser = UnitInputParser(default_units=default_units,
                              convert=convert,
                              check_dims=check_dims,
+                             copy=copy,
                              registry=registry)
     return parser.parse(in_)
 
 
-def _get_units(q) -> unyt.Unit:
-    """Get the units of an object."""
-    try:
-        units = q.units
-    except AttributeError:
-        units = unyt.dimensionless
-    return units
-
-
-def _check_dimensions(a, b):
-    units_a = _get_units(a)
-    units_b = _get_units(b)
-    dim_a = units_a.dimensions
-    dim_b = units_b.dimensions
-    if dim_a != dim_b:
-        raise UnitConversionError(units_a, dim_a, units_b, dim_b)
-
-
 def convert(value, units: UnitLike, registry: unyt.UnitRegistry = None):
     """Convert an input value to the given units, and return a bare quantity.
 
